/*
 * Copyright (C) 2008 Iowa State University 
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _CONFIGURATION_DIALOG_H_
#define _CONFIGURATION_DIALOG_H_

#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
#pragma interface "configuration_dialog.h"
#endif

/*!
 * Includes
 */

////@begin includes
#include "wx/spinctrl.h"
////@end includes

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxSpinCtrl;
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CONFIGURATIONDIALOG 10004
#define ID_SPINCTRL2 10014
#define ID_TEXTCTRL1 10017
#define ID_SPOOL_BROWSE 10018
#define ID_TEXTCTRL 10016
#define ID_GAMESS_BROWSE 10015
#define ID_LOGS 10007
#define SYMBOL_CONFIGURATIONDIALOG_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxDIALOG_MODAL|wxTAB_TRAVERSAL
#define SYMBOL_CONFIGURATIONDIALOG_TITLE _("GamessQ Configuration")
#define SYMBOL_CONFIGURATIONDIALOG_IDNAME ID_CONFIGURATIONDIALOG
#define SYMBOL_CONFIGURATIONDIALOG_SIZE wxDefaultSize
#define SYMBOL_CONFIGURATIONDIALOG_POSITION wxDefaultPosition
////@end control identifiers
#define ID_SETUP_OK 20003

//! Configuration dialog
/*!
 * This class provides the configuration Dialog for GamessQ.
 *
 * \note Most of the code is generated by DialogBlocks, so it may not be very
 * readable.
 */

class ConfigurationDialog: public wxDialog
{    
    DECLARE_DYNAMIC_CLASS( ConfigurationDialog )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    ConfigurationDialog();
    ConfigurationDialog( wxWindow* parent, wxWindowID id = SYMBOL_CONFIGURATIONDIALOG_IDNAME, const wxString& caption = SYMBOL_CONFIGURATIONDIALOG_TITLE, const wxPoint& pos = SYMBOL_CONFIGURATIONDIALOG_POSITION, const wxSize& size = SYMBOL_CONFIGURATIONDIALOG_SIZE, long style = SYMBOL_CONFIGURATIONDIALOG_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CONFIGURATIONDIALOG_IDNAME, const wxString& caption = SYMBOL_CONFIGURATIONDIALOG_TITLE, const wxPoint& pos = SYMBOL_CONFIGURATIONDIALOG_POSITION, const wxSize& size = SYMBOL_CONFIGURATIONDIALOG_SIZE, long style = SYMBOL_CONFIGURATIONDIALOG_STYLE );

    /// Destructor
    ~ConfigurationDialog();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin ConfigurationDialog event handler declarations

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_SPOOL_BROWSE
    void OnSpoolBrowseClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_GAMESS_BROWSE
    void OnGamessBrowseClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_LOGS
    void OnLogsClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
    void OnOkClick( wxCommandEvent& event );

////@end ConfigurationDialog event handler declarations

////@begin ConfigurationDialog member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end ConfigurationDialog member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin ConfigurationDialog member variables
    wxSpinCtrl* refreshFreqSpinCtrl;
    wxTextCtrl* mSpoolDir;
    wxTextCtrl* mGamessPath;
////@end ConfigurationDialog member variables

	//! Sets the value of the "Refresh Frequency" field
	void SetRefreshFrequency(int refreshFrequency);

	//! Returns the value of the "Refresh Frequency" field.
	int GetRefreshFrequency();

	//! Sets the value of the "Spool Directory" field
	void SetSpoolDir(wxString spoolDir);

	//! Returns the value of the "Spool Directory" field.
	wxString GetSpoolDir();

	//! Sets the value of the "GAMESS Directory" field
	void SetGamessPath(wxString gamessPath);

	//! Returns the value of the "GAMESS Directory" field.
	wxString GetGamessPath();
};

#endif
    // _CONFIGURATION_DIALOG_H_
